from airunner.windows.setup_wizard.base_wizard import BaseWizard
from airunner.windows.setup_wizard.model_setup.stable_diffusion_setup.templates.choose_style_ui import Ui_choose_model_style


class ChooseModelStyle(BaseWizard):
    class_name_ = Ui_choose_model_style


