from airunner.windows.setup_wizard.base_wizard import BaseWizard
from airunner.windows.setup_wizard.installation_settings.templates.install_success_page_ui import \
    Ui_install_success_page


class InstallSuccessPage(BaseWizard):
    class_name_ = Ui_install_success_page
